import java.util.Set;
import java.util.HashSet;

public class NFA {
    protected Set<FSMState> states;
    private FSMState startState;
    private Set<FSMState> finalStates;
    private Set<FSMTransition> transitions;
    private static final char EPSILON = '-';
    private static final char TAB = 9;

    /* Main Constructor */
    public NFA(Set<FSMState> states, FSMState startState, Set<FSMState> finalStates, Set<FSMTransition> transitions) {
        if (!states.contains(startState)) {
            throw new IllegalArgumentException("The start state must be in the NFA.");
        }

        if (!states.containsAll(finalStates)) {
        	throw new IllegalArgumentException("The final states must be in the NFA.");
        }
        
        for (FSMTransition t : transitions) {
        	if (!states.contains(t.getSource()) || !states.contains(t.getDestination())) {
        		throw new IllegalArgumentException("The transitions may only use states in the NFA.");
        	}
        }

        this.states = states;
        this.startState = startState;
        this.finalStates = finalStates;
        this.transitions = transitions;
    }

    /* Copy Constructor */
    public NFA(NFA n) {
        this(new HashSet<FSMState>(n.states), n.startState, new HashSet<FSMState>(n.finalStates), new HashSet<FSMTransition>(n.transitions));
    }

    /* @returns a new NFA that accepts any single character. */
    public static NFA dot() {
        HashSet<FSMState> states = new HashSet<FSMState>();
		HashSet<FSMState> finalStates = new HashSet<FSMState>();
        FSMState startState = new FSMState();
        FSMState acceptState = new FSMState();

        states.add(startState);
        states.add(acceptState);

        finalStates.add(acceptState);

		HashSet<FSMTransition> transitions = new HashSet<FSMTransition>();

        for (char c=' '; c <= '~'; c++) {
            transitions.add(new FSMTransition(c, startState, acceptState));
        }
        transitions.add(new FSMTransition(TAB, startState, acceptState));

        return new NFA(states, startState, finalStates, transitions);
    }


    /* @returns true if and only if the NFA accepts s. */
    public boolean read(String s) {
        /* TODO: You need to implement this! */
        return false;
    }
}
